﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using WindowsUtilities;

namespace CTNet
{
    public enum verifyFile { fileOk, exception, nullEmptyPath, nullEmptyFileName, nullWorkFolder, notDocFile, noPortFile, noZipFile };

    // Where are box selections going?
    public enum boxSelDest { off, chkBoxCtrl, dfRetriever, document };

    // Things we tell the form about:
    public enum ctEvent { connected, connectFail, boxSelReq, boxSelOff, shutdown, saveDoc, docLock };

    // A custom EventArgs class to pass ctClient info to main form.
    public class CtClientArgs : EventArgs
    {
        public ctEvent eventType { get; set; }
        public string eventMsg { get; set; }
        //public string eventToolId { get; set; }
    }

    public class CTClient
    {
        // Create the calc tool client event itself:
        public event EventHandler<CtClientArgs> CtClientEvent;

        // Where are box selections going?
        public boxSelDest boxSelClient;

        private static bool bExiting = false;
        private static bool bConnected = false;

        /*
         * Saw a problem while testing trying to connect to
         * Calc Tool Server when I knew it was not running. Got an exception in
         * the ConnectCallback() routine which caused StartClient() to hang on
         * the connectDone.WaitOne() call since there was no connectDone.Set()
         * called from ConnectCallback(). So, I am calling connectDone.Set()
         * in a finally block in ConnectCallback() along with setting this
         * exception flag and will check it in StartClient() to make sure we
         * have a 'real' connectDone.Set() signal.
         */
        public bool connException = false;

        private static int port = 0;

        private static Socket client = null;

        // This is the arg[0] from Visual Studio Code launch.
        //private static string vscFile = null;
        public string vscFile = null;

        public static string vscFileNameNoExt = null;

        // This is the file name and extension from arg[0].
        private static string workFile = null;

        // This is the directory information from arg[0].
        private static string workFolder = null;

        // This is the zip path and file name for workFile - ext is .zip.
        //private static string zipFile = null;
        public string zipFile = null;

        // Location of Calc Tool Server port number file so client tool can connect.
        private static string portFile = null;

        private static IntPtr _hWnd = IntPtr.Zero;

        private static CTMessage gMsg = new CTMessage();

        // Thread signal. Probably will not use some of these..... 
        private static ManualResetEvent allDone = new ManualResetEvent(false);
        private static ManualResetEvent msgDone = new ManualResetEvent(false);

        private static ManualResetEvent sendMsgDone = new ManualResetEvent(false);
        private static ManualResetEvent receiveMsgDone = new ManualResetEvent(false);

        private static ManualResetEvent sendDone = new ManualResetEvent(false);
        private static ManualResetEvent connectDone = new ManualResetEvent(false);
        private static ManualResetEvent connectAckDone = new ManualResetEvent(false);

        public CTClient(VSCFileInfo vscFileInfo)
        {
            // Doing this currently, actual Calc Helper may differ:
            boxSelClient = boxSelDest.document;

            /*
             * I think this is called 'normalizing' the file path,
             * so c:\blah\blah and c:/blah/blah are comparable
             * to each other. Needed when we get a file 
             * open/close/save message from Vscode.
             */
            vscFile = Path.GetFullPath(vscFileInfo.fileTmp).ToLower();

            vscFileNameNoExt = vscFileInfo.fileNameNoExtTmp;
            workFolder = vscFileInfo.workFolderTmp;
            workFile = vscFileInfo.workFileTmp;
            zipFile = vscFileInfo.zipFileTmp;
            portFile = vscFileInfo.portFileTmp;
        }

        // Trigger the calc tool client event:
        protected virtual void OnCtClientEvent(CtClientArgs e)
        {
            if (CtClientEvent != null)
            {
                CtClientEvent(this, e);
            }
        }

        public bool IsConnected()
        {
            bool result = false;

            if (client != null && client.Connected == true)
            {
                result = true;
            }

            return result;
        }

        public void StopClient()
        {
            bExiting = true;
            receiveMsgDone.Set();

            if (client != null && client.Connected == true)
            {
                try
                {
                    client.Shutdown(SocketShutdown.Both);
                    client.Close();
                }
                catch
                {

                }
            }
        }

        public void StartClient()
        {
            CtClientArgs ctClientArgs = new CtClientArgs();

            // Connect to Visual Studio Code server.
            if (ReadPortNumber() == false)
            {
                MessageBox.Show("  Could not read .calctool port number file.\n\n  Calc Helper launch will be aborted.", "Calc Tool Server Connection Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);

                ctClientArgs.eventType = ctEvent.connectFail;
            }
            else
            {
                try
                {
                    // Establish the remote endpoint for the socket.  
                    IPAddress ipAddress = IPAddress.Loopback;
                    IPEndPoint remoteEP = new IPEndPoint(ipAddress, port);

                    // Create a TCP/IP socket.  
                    client = new Socket(ipAddress.AddressFamily, SocketType.Stream, ProtocolType.Tcp);

                    // Connect to the remote endpoint.  
                    client.BeginConnect(remoteEP, new AsyncCallback(ConnectCallback), client);

                    connectDone.WaitOne();

                    // Debug:
                    // Added this if (..) { .. } else { ... } to see if we can abort and close
                    // this application if ConnectCallback() has an exception.
                    if (connException)
                    {
                        //MessageBox.Show("  We have a connException!", "Calc Helper-StartClient():", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        //client = null;
                        ctClientArgs.eventType = ctEvent.connectFail;
                    }
                    else
                    {
                        // Send connect message to Calc Tool Server.
                        SendConnectMsg();

                        if (sendMsgDone.WaitOne(5000) == true)
                        {
                            // Now we expect the Connection Acknowledgment.
                            //MessageBox.Show("  Waiting for connection acknowledgment from Calc Tool Server.", "Calc Helper-StartClient():", MessageBoxButtons.OK, MessageBoxIcon.Information);

                            receiveMsgDone.Reset();

                            ReceiveAsync(client);

                            if (receiveMsgDone.WaitOne(5000) == true)
                            {
                                // Signal main form that we are connected to Calc Tool Server.
                                ctClientArgs.eventType = ctEvent.connected;

                                //MessageBox.Show("  Calc Tool Server has acknowledged our connection!.", "Calc Helper-StartClient():", MessageBoxButtons.OK, MessageBoxIcon.Information);

                                // Loop for VSC messages:
                                //while (!bExiting)
                                //{
                                //    receiveMsgDone.Reset();

                                //    ReceiveAsync(client);

                                //    receiveMsgDone.WaitOne();
                                //}
                            }
                            else
                            {
                                MessageBox.Show("  Timed out connecting to Calc Tool Server.\n\n  Calc Helper launch will be aborted.", "Calc Tool Server Connection Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);

                                ctClientArgs.eventType = ctEvent.connectFail;
                            }
                        }
                        else
                        {
                            MessageBox.Show("  Timed out connecting to Calc Tool Server.\n\n  Calc Helper launch will be aborted.", "Calc Tool Server Connection Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);

                            ctClientArgs.eventType = ctEvent.connectFail;
                        }
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.ToString(), "Calc Tool Server Connection Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    ctClientArgs.eventType = ctEvent.connectFail;
                }
            }

            OnCtClientEvent(ctClientArgs);
        }

        public void SetVscodeFocus()
        {
            string folder = workFolder.Substring(workFolder.LastIndexOf('\\') + 1);
            string wndwTitle = string.Format("{0} - {1} - Visual Studio Code", workFile, folder);

            //MessageBox.Show(string.Format("Looking for: {0}", wndwTitle), "SetVscodeFocus():", MessageBoxButtons.OK, MessageBoxIcon.Information);

            try
            {
                IEnumerable<IntPtr> hWnd = WinUtils.FindWindowsWithText(wndwTitle);

                if (hWnd.Count() > 0)
                {
                    //MessageBox.Show(string.Format("hWnd.Count() = {0}", hWnd.Count()), "Entered: if (hWnd.Count() > 0)", MessageBoxButtons.OK, MessageBoxIcon.Information);

                    WinUtils.SetForegroundWindow(hWnd.ElementAt(0));
                }
                else
                {
                    MessageBox.Show(string.Format("  Could not set Visual Studio Code Focus for:\n\n  {0}\n\n  You will need to manually mouse click the\n  Visual Studio Code window to give it focus.", wndwTitle), "SetVscodeFocus() Error:", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "CTClient-SetVscodeFocus():", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        public void SendBoxSelections(string boxSel)
        {
            if (client != null && client.Connected == true)
            {
                CTMessage msg = new CTMessage();

                msg.Data = boxSel;

                // Who are we sending these boxes to?
                switch (boxSelClient)
                {
                    case (boxSelDest.chkBoxCtrl):

                        msg.Command = "BoxSelCBC";
                        msg.FileName = vscFileNameNoExt;
                        break;

                    case (boxSelDest.dfRetriever):

                        msg.Command = "BoxSelDFR";
                        msg.FileName = vscFileNameNoExt;
                        break;

                    case (boxSelDest.document):

                        msg.Command = "BoxSelDOC";

                        // Will probably switch to the clipboard.
                        // I think we need to now [the server needs it this way].
                        Clipboard.SetText(boxSel, TextDataFormat.Text);
                        SetVscodeFocus();
                        break;
                }

                SendAsync(msg);
            }
        }

        public void GotoDataflow(string dfStr)
        {
            if (client != null && client.Connected == true)
            {
                CTMessage msg = new CTMessage();

                msg.Data = dfStr;

                msg.Command = "GotoDataflow";

                Clipboard.SetText(dfStr, TextDataFormat.Text);
                SetVscodeFocus();

                SendAsync(msg);
            }
        }

        public void GotoCode(string codeStr)
        {
            if (client != null && client.Connected == true)
            {
                CTMessage msg = new CTMessage();

                msg.Data = codeStr;

                msg.Command = "GotoCode";

                Clipboard.SetText(codeStr, TextDataFormat.Text);
                SetVscodeFocus();

                SendAsync(msg);
            }
        }

        // From Joe's SBClient:
        //private static void SendConnectMsg()
        public void SendConnectMsg()
        {
            CTMessage msg = new CTMessage();
            msg.Command = "Connect";

            SendAsync(msg);
        }

        // Quick throw together to see if two tools can communicate...
        //private static void SendMessage(string msgType)
        public void SendMessage(string msgType)
        {
            if (msgType == "BoxAddDelAck")
            {
                CTMessage msg = new CTMessage();
                msg.Command = "BoxAddDelAck";

                SendAsync(msg);
            }
        }

        // From Joe's SBClient:
        //private static void SendAsync(CTMessage msg)
        public void SendAsync(CTMessage msg)
        {
            if (client != null && client.Connected)
            {
                msg.Id = "CH";
                string json = JsonConvert.SerializeObject(msg);

                byte[] bytes = Encoding.ASCII.GetBytes(json);

                byte[] head = BitConverter.GetBytes(bytes.Length);
                byte[] dmsg = new byte[bytes.Length + head.Length];
                head.CopyTo(dmsg, 0);
                bytes.CopyTo(dmsg, head.Length);

                // Begin sending the data to the remote device.  
                client.BeginSend(dmsg, 0, dmsg.Length, 0, new AsyncCallback(SendCallback), client);
            }
        }

        // From Joe's SBClient:
        //private static void SendCallback(IAsyncResult ar)
        public void SendCallback(IAsyncResult ar)
        {
            try
            {
                // Retrieve the socket from the state object.  
                Socket client = (Socket)ar.AsyncState;

                // Complete sending the data to the remote device.  
                int bytesSent = client.EndSend(ar);
                Console.WriteLine("Sent {0} bytes to server.", bytesSent);

                // Signal that all bytes have been sent.  
                sendMsgDone.Set();
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }
        }

        //private static bool ReadPortNumber()
        public bool ReadPortNumber()
        {
            bool ret = false;

            try
            {
                string text = File.ReadAllText(portFile, Encoding.UTF8);
                ret = int.TryParse(text, out port);
            }
            catch (Exception ex)
            {
                ret = false;
                MessageBox.Show(ex.ToString(), "CTClient-ReadPortNumber():", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            return ret;
        }

        //private static void ConnectCallback(IAsyncResult ar)
        public void ConnectCallback(IAsyncResult ar)
        {
            CtClientArgs ctClientArgs = new CtClientArgs();

            try
            {
                // Retrieve the socket from the state object.  
                Socket client = (Socket)ar.AsyncState;

                // Complete the connection.  
                client.EndConnect(ar);

                //MessageBox.Show(string.Format("Socket connected to {0}", client.RemoteEndPoint.ToString()), "Connected:", MessageBoxButtons.OK, MessageBoxIcon.Information);

                // Signal that the connection has been made.  
                connectDone.Set();
            }
            catch (Exception ex)
            {
                /*
                 * Discovered tricky error condition while purposefully launching Calc Helper
                 * when the Calc Tool Server is not running. The form is blocked by the connectDone.WaitOne()
                 * call in StartClient(), so doing this and calling connectDone.Set() to kickstart execution
                 * but will be checking for connException to see if it was this situation.
                 */
                connException = true;

                MessageBox.Show(ex.ToString(), "CTClient-ConnectCallback() - Calc Helper Launch Will Be Aborted:", MessageBoxButtons.OK, MessageBoxIcon.Error);

                connectDone.Set();
            }
            // Debug - Start:
            //finally
            //{
            //    //connException = true;
            //    //connectDone.Set();
            //}
            // Debug - Stop.
        }

        //private static void ReceiveAsync(Socket skt)
        public void ReceiveAsync(Socket skt)
        {
            if (skt != null && skt.Connected)
            {
                InitStateObject isa = new InitStateObject();
                isa.workSocket = skt;

                try
                {
                    skt.BeginReceive(isa.LenBuffer, 0, isa.LenBuffer.Length, SocketFlags.None, new AsyncCallback(InitAsyncReadCallback), isa);
                }
                //catch (Exception ex)
                //{
                //    MessageBox.Show(ex.ToString(), "CTClient-ReceiveAsync():", MessageBoxButtons.OK, MessageBoxIcon.Error);
                //}
                catch
                {

                }
            }
        }

        //private static void InitAsyncReadCallback(IAsyncResult ar)
        public void InitAsyncReadCallback(IAsyncResult ar)
        {
            try
            {
                InitStateObject isa = ar.AsyncState as InitStateObject;

                Socket client = isa.workSocket;

                int rcvd = client.EndReceive(ar);

                if (rcvd != 4)
                    throw new Exception();

                int to_recv = BitConverter.ToInt32(isa.LenBuffer, 0);

                // Create the state object.  
                StateObject state = new StateObject(to_recv);
                state.workSocket = client;
                state.toReceive = to_recv;

                client.BeginReceive(state.buffer, 0, state.buffer.Length, SocketFlags.None, new AsyncCallback(AsyncReadCallback), state);
            }
            //catch (Exception ex)
            //{
            //    MessageBox.Show(ex.ToString(), "CTClient-InitAsyncReadCallback():", MessageBoxButtons.OK, MessageBoxIcon.Error);
            //}
            catch
            {

            }
        }

        //public static void AsyncReadCallback(IAsyncResult ar)
        public void AsyncReadCallback(IAsyncResult ar)
        {
            StateObject state = ar.AsyncState as StateObject;
            Socket client = state.workSocket;

            int rcvd = client.EndReceive(ar);

            if (rcvd <= 0)
                return;

            state.BufStream.Write(state.buffer, 0, rcvd);

            state.toReceive -= rcvd;

            if (state.toReceive > 0)
            {
                Array.Clear(state.buffer, 0, state.buffer.Length);
                client.BeginReceive(state.buffer, 0, state.buffer.Length, SocketFlags.None, new AsyncCallback(AsyncReadCallback), state);
                return;
            }

            state.BufStream.Position = 0;

            string message = Encoding.ASCII.GetString(state.BufStream.ToArray());

            state.BufStream.Close();

            state.message = JsonConvert.DeserializeObject<CTMessage>(message);

            //MessageBox.Show(String.Format("Id: {0}, Command: {1}, FileName: {2}, Status: {3}", state.message.Id, state.message.Command, state.message.FileName, state.message.Status), "CTClient-AsyncReadCallback():", MessageBoxButtons.OK, MessageBoxIcon.Information);

            ProcessMessage(client, state.message);

            /*
             * Trying to see if two way communication works with this commented out.
             * Connect to CT server->receive ConnectAck;
             * Receive BoxAddDel Report command from BAD -> send BoxAddDelAck to BAD;
             * Send BoxSel info to CT server;
             */
            // 06/04/19 Just added this..so far CBC request for boxes times out...
            receiveMsgDone.Reset();

            ReceiveAsync(client);
        }

        //public static void ProcessMessage(Socket client, CTMessage message)
        public void ProcessMessage(Socket client, CTMessage message)
        {
            CtClientArgs ctClientArgs;
            CTMessage msg;

            //MessageBox.Show("Message: " + message.Command, "CTClient-ProcessMessage():", MessageBoxButtons.OK, MessageBoxIcon.Information);
            //Debug.Print("Message: " + message.Command);

            switch (message.Command)
            {
                case ("ConnectAck"):

                    if (message.Id.StartsWith("VSC") == true)
                    {
                        receiveMsgDone.Set();

                        //MessageBox.Show("Calc Tool Server has acknowledged the connection!", "CH ProcessMessage():", MessageBoxButtons.OK, MessageBoxIcon.Information);

                        bConnected = true;

                        //WinUtils.SendCopyDataMessage(_hWnd, "editor_connected:");
                        ////string sb_id = message.FileName.Substring(message.FileName.LastIndexOf('\\') + 1);

                        ////WinUtils.PostMessage(_hWnd, WinUtils.WM_COPYDATA, 0, 0);

                        ////WinUtils.PostMessage(_hWnd, WinUtils.WM_CLOSE, 0, 0);
                        ////WinUtils.SendCopyDataMessage(_hWnd, "Hello World");
                    }

                    break;

                case ("CBCBoxSel"):

                    //MessageBox.Show("CBC wants some boxes", "CH ProcessMessage():", MessageBoxButtons.OK, MessageBoxIcon.Information);

                    /*
                     * Check Box Control is requesting check box selections.
                     * We must make sure both tools are connected to the same file.
                     */
                    msg = new CTMessage();

                    if (message.FileName == vscFileNameNoExt)
                    {
                        msg.Command = "CBCBoxSelAck";

                        boxSelClient = boxSelDest.chkBoxCtrl;

                        // Tell the form about this.
                        ctClientArgs = new CtClientArgs();
                        ctClientArgs.eventType = ctEvent.boxSelReq;
                        OnCtClientEvent(ctClientArgs);
                    }
                    else
                    {
                        //MessageBox.Show(string.Format("CBC wants some boxes but file names differ\n  message.FileName = {0}\n  CH file name = {1}", message.FileName, vscFileNameNoExt), "CH ProcessMessage():", MessageBoxButtons.OK, MessageBoxIcon.Information);

                        msg.Command = "CBCBoxSelFileNack";
                        msg.FileName = vscFileNameNoExt;
                    }

                    SendAsync(msg);

                    break;

                case ("CBCBoxSelOff"):

                    /*
                     * Check Box Control is no longer wants check box selections.
                     * Tell the form about this.
                     */
                    ctClientArgs = new CtClientArgs();

                    ctClientArgs.eventType = ctEvent.boxSelOff;
                    OnCtClientEvent(ctClientArgs);

                    break;

                case ("BoxAddDel"):

                    // Acknowledge the message back to Box Adds/Deletes tool.
                    SendMessage("BoxAddDelAck");

                    MessageBox.Show(string.Format("Box Add/Delete Report Message Received:\n\n  {0}", message.Data), "CH ProcessMessage():", MessageBoxButtons.OK, MessageBoxIcon.Information);

                    break;

                case ("Disconnect-BAD"):

                    MessageBox.Show("BAD has disconnected from tool server.", "CH ProcessMessage():", MessageBoxButtons.OK, MessageBoxIcon.Information);

                    break;

                case ("Disconnect-CBC"):

                    //MessageBox.Show("CBC has disconnected from tool server.", "CH ProcessMessage():", MessageBoxButtons.OK, MessageBoxIcon.Information);

                    /*
                     * Check Box Control has disconnected from the Calc Tool Server.
                     * Tell the form about this if selecting boxes for Check Box Control.
                     */
                    if (boxSelClient == boxSelDest.chkBoxCtrl)
                    {
                        ctClientArgs = new CtClientArgs();

                        ctClientArgs.eventType = ctEvent.boxSelOff;
                        OnCtClientEvent(ctClientArgs);
                    }

                    break;

                case ("Disconnect-DFR"):

                    //MessageBox.Show("DFR has disconnected from tool server.", "CH ProcessMessage():", MessageBoxButtons.OK, MessageBoxIcon.Information);

                    break;

                case ("OpenDoc"):

                    MessageBox.Show(String.Format("Text Document Opened In Visual Studio Code:\n\n  {0}", message.FileName), "CH ProcessMessage():", MessageBoxButtons.OK, MessageBoxIcon.Information);

                    break;

                case ("CloseDoc"):

                    MessageBox.Show(String.Format("Text Document Closed In Visual Studio Code:\n\n  {0}", message.FileName), "CH ProcessMessage():", MessageBoxButtons.OK, MessageBoxIcon.Information);

                    break;

                case ("Pong"):

                    Console.WriteLine("Pong");
                    gMsg.Command = "Pong";
                    msgDone.Set();

                    break;

                case ("QueryDocDirty"):

                    //Console.WriteLine("QueryDocDirty");
                    //gMsg.Status = message.Status;
                    //WinUtils.SendCopyDataMessage(_hWnd, "query_doc_dirty:" + message.Status);
                    //msgDone.Set();

                    break;

                case ("QueryAnyDocDirty"):

                    //Console.WriteLine("QueryDocDirty");
                    //gMsg.Status = message.Status;
                    //WinUtils.SendCopyDataMessage(_hWnd, "query_any_doc_dirty:" + message.Status);
                    //msgDone.Set();

                    break;

                case ("Disconnecting"):

                    if (message.Id.StartsWith("VSC") == true)
                    {
                        // Vscode is going down, bye bye time...
                        ctClientArgs = new CtClientArgs();
                        ctClientArgs.eventType = ctEvent.shutdown;
                        OnCtClientEvent(ctClientArgs);
                    }

                    break;

                case ("SaveDoc"):

                    MessageBox.Show(String.Format("Text Document Saved In Visual Studio Code:\n\n  {0}\n\nvscFile: {1}", Path.GetFullPath(message.FileName).ToLower(), vscFile), "CH ProcessMessage():", MessageBoxButtons.OK, MessageBoxIcon.Information);

                    ctClientArgs = new CtClientArgs();

                    if (Path.GetFullPath(message.FileName).ToLower() == vscFile)
                    {
                        ctClientArgs.eventType = ctEvent.saveDoc;
                    }
                    else
                    {
                        // Trouble in paradise....
                        MessageBox.Show(String.Format("Text Document Saved In Visual Studio Code:\n  {0}\n\n  Does not match Calc Helper file:\n  {1}\n\n  Calc Helper is suspending interaction with\n  Visual Studio Code.", Path.GetFullPath(message.FileName).ToLower(), vscFile), "Mismatching Files:", MessageBoxButtons.OK, MessageBoxIcon.Error);

                        ctClientArgs.eventType = ctEvent.docLock;
                    }

                    OnCtClientEvent(ctClientArgs);

                    break;
            }
        }

        //public bool SetVSCFileInfo()
        //public bool SetVSCFileInfo(string vscEditorFile)
        //{
        //    bool ret = true;

        //    try
        //    {
        //        vscFile = vscEditorFile;

        //        workFile = Path.GetFileName(vscEditorFile);

        //        /*
        //         * This will be null if: vscEditorFile denotes a root directory or is null, but the
        //         * caller made sure vscEditorFile is not null. This will be the empty string if: 
        //         * vscEditorFile does not contain directory information.
        //         */
        //        workFolder = Path.GetDirectoryName(vscEditorFile);

        //        if (workFolder == null)
        //        {
        //            /*
        //             * So ars[0] must denote a root directory.
        //             * This will either be (the root directory) or (the empty string if args[0]
        //             * does not contain root directory information-not sure if this will be encountered).
        //             */
        //            workFolder = Path.GetPathRoot(vscEditorFile);
        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        ret = false;
        //        MessageBox.Show(ex.ToString(), "CTClient-SetupFileInfo():", MessageBoxButtons.OK, MessageBoxIcon.Error);
        //    }

        //    return ret;
        //}

        //public bool SetPortFile()
        //{
        //    bool ret = false;

        //    try
        //    {
        //        // Verify existance of .calctool port number file:
        //        portFile = Path.Combine(workFolder, ".calctools");

        //        if (File.Exists(portFile))
        //        {
        //            ret = true;
        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        ret = false;
        //        MessageBox.Show(ex.ToString(), "CTClient-SetPortFile():", MessageBoxButtons.OK, MessageBoxIcon.Error);
        //    }

        //    return ret;
        //}

        /*
         * Active text editor in VSC has changed. Check if current text editor's
         * file is a document's file and if we can locate the associated zip.
         * If so, then offer to have Calc Helper link to this new file.
         */
    //    public bool VerifyFileInfo(string vscItemPath)
    //    {
    //        bool zipFound = false;
    //        string zipFileName;

    //        try
    //        {
    //            if (!string.IsNullOrEmpty(vscItemPath))
    //            {
    //                vscFileTmp = vscItemPath;
    //                workFileTmp = Path.GetFileName(vscItemPath);

    //                if (!string.IsNullOrEmpty(workFileTmp))
    //                {
    //                    if (Regex.Match(workFileTmp.ToLower(), @"^[a-z]{4}\d{4}\.(rb|rbw|js)$").Success)
    //                    {
    //                        zipFileName = Path.GetFileNameWithoutExtension(workFileTmp) + ".zip";

    //                        workFolderTmp = GetFolder(vscItemPath);

    //                        if (workFolderTmp != null)
    //                        {
    //                            zipFileTmp = Path.Combine(workFolderTmp, zipFileName);

    //                            zipFound = File.Exists(zipFileTmp);

    //                            if (!zipFound)
    //                            {
    //                                zipFileTmp = Path.Combine(pdfCurDir, zipFileName);
    //                                zipFound = File.Exists(zipFileTmp);
    //                            }

    //                            if (!zipFound)
    //                            {
    //                                vscFileTmp = null;
    //                                workFileTmp = null;
    //                                workFolderTmp = null;
    //                                zipFileTmp = null;
    //                            }
    //                        }
    //                    }
    //                }
    //            }
    //        }
    //        catch (Exception ex)
    //        {
    //            zipFound = false;
    //            MessageBox.Show(ex.ToString(), "CTClient-VerifyFileInfo():", MessageBoxButtons.OK, MessageBoxIcon.Error);
    //        }

    //        return zipFound;
    //    }

    //    public bool SetZipFile(string vscEditorFile)
    //    {
    //        bool ret = false;
    //        bool zipFound = false;
    //        string fileName = null;
    //        string fileNameNoExt = null;
    //        string pathNoFileName = null;
    //        string zipFile = null;

    //        try
    //        {
    //            fileNameNoExt = Path.GetFileNameWithoutExtension(vscEditorFile);
    //            pathNoFileName = Path.GetDirectoryName(vscEditorFile);

    //            //MessageBox.Show(String.Format("File Name [no ext]: {0}", fileNameNoExt));
    //            //MessageBox.Show(String.Format("Just the path [no file name]: {0}", pathNoFileName));

    //            zipFile = pathNoFileName + fileNameNoExt + ".zip";
    //            zipFound = File.Exists(zipFile);

    //            if (zipFound)
    //            {
    //                pdfCurDir = pathNoFileName;
    //            }
    //            else
    //            {
    //                zipFile = pdfCurDir + fileNameNoExt + ".zip";
    //                zipFound = File.Exists(zipFile);
    //            }
    //        }
    //        catch (Exception ex)
    //        {
    //            ret = false;
    //            MessageBox.Show(ex.ToString(), "CTClient-SetDocZipFile():", MessageBoxButtons.OK, MessageBoxIcon.Error);
    //        }

    //        return ret;
    //    }
    }

    public class VSCFileInfo
    {
        public string fileTmp = null;
        public string fileNameNoExtTmp = null;
        public string workFolderTmp = null;
        public string workFileTmp = null;
        public string zipFileTmp = null;
        public string portFileTmp = null;

        public verifyFile verifyResult;

        /*
         * One of the following has happened:
         * 
         * A) Active text editor in VSC has changed. We need to first verify
         * associated file information before we decide if this file should be
         * offered to the user for connecting to instead of the current file.
         * doPort is 'false'.
         * 
         * B) Calc Helper has been launched from Visual Studio with this file
         * argument - we must verify before we can continue with the launch process.
         * doPort is 'true'.
         */
        public void VerifyFileInfo(string vscItemPath, bool doPort)
        {
            string zipFileName;

            verifyResult = verifyFile.fileOk;

            try
            {
                if (string.IsNullOrEmpty(vscItemPath))
                {
                    verifyResult = verifyFile.nullEmptyPath;
                }
                else
                {
                    fileTmp = vscItemPath;
                    workFileTmp = Path.GetFileName(vscItemPath);

                    if (string.IsNullOrEmpty(workFileTmp))
                    {
                        verifyResult = verifyFile.nullEmptyFileName;
                    }
                    else 
                    {
                        if (Regex.Match(workFileTmp.ToLower(), @"^[a-z]{4}\d{4}\.(rb|rbw|js)$").Success)
                        {
                            workFolderTmp = GetFolder(vscItemPath);

                            if (workFolderTmp != null)
                            {
                                if (doPort)
                                {
                                    // Verify existance of .calctool port number file:
                                    portFileTmp = Path.Combine(workFolderTmp, ".calctools");

                                    if (!File.Exists(portFileTmp))
                                    {
                                        verifyResult = verifyFile.noPortFile;
                                        return;
                                    }
                                }

                                fileNameNoExtTmp = Path.GetFileNameWithoutExtension(workFileTmp).ToUpper();
                                zipFileName = fileNameNoExtTmp + ".zip";
                                zipFileTmp = Path.Combine(workFolderTmp, zipFileName);

                                if (!File.Exists(zipFileTmp))
                                {
                                    zipFileTmp = Path.Combine(CalcHelper.Form1.pdfCurDir, zipFileName);

                                    if (!File.Exists(zipFileTmp))
                                    {
                                        verifyResult = verifyFile.noZipFile;
                                    }
                                }
                            }
                            else
                            {
                                verifyResult = verifyFile.nullWorkFolder;
                            }
                        }
                        else
                        {
                            verifyResult = verifyFile.notDocFile;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                verifyResult = verifyFile.exception;
                MessageBox.Show(ex.ToString(), "CTClient-VerifyFileInfo():", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        public string GetFolder(string vscItemPath)
        {
            string folder = null;

            // Caller has ensured that vscEditorFile is not null.

            folder = Path.GetDirectoryName(vscItemPath);

            if (folder == null)
            {
                // Root path or the empty string.
                folder = Path.GetPathRoot(vscItemPath);
            }

            return folder;
        }

        //public string pdfCurDir { get; set; }
    }

    public class StateObject
    {
        public Socket workSocket = null;
        public byte[] buffer; // = new byte[BufferSize];

        public int toReceive;
        public MemoryStream BufStream;
        public CTMessage message;

        public StateObject(int toRec)
        {
            toReceive = toRec;
            BufStream = new MemoryStream(toRec);
            buffer = new byte[toRec];
        }
    }

    public class InitStateObject
    {
        public Socket workSocket = null;
        public byte[] LenBuffer = new byte[4];
    }


    public class CTMessage
    {
        public string Id { get; set; }
        public string Command { get; set; }
        public string FileName { get; set; }
        public string Status { get; set; }
        public string Data { get; set; }
        public Int32 hWnd;

        public CTMessage()
        {
            Id = "";
            Command = "";
            FileName = "";
            Status = "";
            Data = "";
            hWnd = 0;
        }
    }
}